from math import  atan, radians,  degrees
import os
from time import sleep
import RPi.GPIO as GPIO

VERTICAL_CAPTEUR_K5= 23.7
HAUTEUR_CAPTEUR_K5 = 15.7
FOCALE = 18.0
TAUX_DE_RECOUVREMENT = 0.30
verticale_angle_de_la_scene = 360.0
hauteur_angle_de_la_scene= 180.0
verticale_fov = 2.0*atan(VERTICAL_CAPTEUR_K5/2.0/FOCALE)
hauteur_fov = 2.0*atan(HAUTEUR_CAPTEUR_K5/2.0/FOCALE)
verticale_angle_de_rotation = verticale_fov * ( 1 - TAUX_DE_RECOUVREMENT)
hauteur_angle_de_rotation = hauteur_fov * (1 - TAUX_DE_RECOUVREMENT)

veticale_nombre_de_photo = int(radians(verticale_angle_de_la_scene/verticale_angle_de_rotation)) + 1
hauteur_nombre_de_photo = int(radians(hauteur_angle_de_la_scene/hauteur_angle_de_rotation)) + 1

class TetePanoramique(object):
    def __init__(self):
        DENTS_ROUE_LEGO = 56.0
        DENTS_ROUE_ROUGE = 40.0
        self.axe_vertical = DENTS_ROUE_LEGO
        self.axe_horizontal = DENTS_ROUE_ROUGE

class  CarteULN2803A(object):
    def __init__(self):
        # Use BCM GPIO references
        # instead of physical pin numbers
        GPIO.setmode(GPIO.BCM)
        self.brochage = [[17,  18,  27,  22], [23, 24, 25, 4]]
        for broche in range(2):
            for adresse_gpio in range(4):
                GPIO.setup(self.brochage[broche][adresse_gpio], GPIO.OUT)
                GPIO.output(self.brochage[broche][adresse_gpio], 0)
          
    def arretPropre(self):
        GPIO.cleanup()
        
class Connecteur(CarteULN2803A): 
    def __init__(self, numero_brochage):
        CarteULN2803A.__init__(self)
        self.phase = [self.brochage[numero_brochage][0], self.brochage[numero_brochage][1] , self.brochage[numero_brochage][2] , self.brochage[numero_brochage][3]]
        
    def envoyerImpulsion(self, numero_phase, numero_sequence ):
        GPIO.output(self.phase[numero_phase],  1 - numero_sequence % 2)
                
class Moteur28BYJ_48(object):
    def __init__(self,connecteur, axe_reduction):
        #donnee constructeur URL: http://www.raspberrypi-spy.co.uk/wp-content/uploads/2012/07/Stepper-Motor-28BJY-48-Datasheet.pdf
        #phase1 = 0 -- phase2 = 1  -- phase3 = 2  -- phase4 = 3
        self.SEQUENCE_PHASE = [[3, 0, 2, 3, 1, 2, 0, 1, 3] , [3, 2,  3,  1,  2,  0, 1, 3 , 0]]
        self.SEQUENCE_LED= [["0 ---* phase 4", "1 ---* phase 1",  "2 --** phase 3",  "3 --*- phase 4",  "4 -**- phase 2",  "5 -*-- phase 3",  "6 **-- phase 1",  "7 *--- phase 2",  "8 *--* phase 4"] , ["0 ---* phase 4", "1 ---* phase 3",  "2 --** phase 4",  "3 --*- phase 2",  "4 -**- phase 3",  "5 -*-- phase 1",  "6 **-- phase 2",  "7 *--- phase 4",  "8 *--* phase 1"]]
        REDUCTION_ENGRENAGE = 64.0
        NOMBRE_IMPULSION_PAR_TOUR = 64.0   
        #rampe acceleration
        self.tour_mn_apres_reduction_mini = 3.0
        self.tour_mn_apres_reduction_maxi = 7.0
        angle_de_la_rampe_variation_vitesse = 2  
        self.rapport_reduction_moteur = REDUCTION_ENGRENAGE * NOMBRE_IMPULSION_PAR_TOUR
        self.rapport_reduction =  self.rapport_reduction_moteur* axe_reduction
        self.max_impulsion_vitesse_constante = int((self.rapport_reduction * abs(angle_de_la_rampe_variation_vitesse)) / 360) 
        #mise sous tension du moteur
        self.mon_connecteur = connecteur
        self.mon_connecteur.envoyerImpulsion(self.SEQUENCE_PHASE[0][0],0 )
        sleep(1)
        self.numero_sequence_depart = 1
        self.numero_sequence_depart_LED = 1
      
    def positionnerTeteAngleEnDegres (self,  angle):
        numero_sequence = self.numero_sequence_depart
        self.total_impulsion_angle = int((self.rapport_reduction * abs(angle)) / 360.0)
        if angle > 0:
            sens_rotation = 0
            for impulsion in range (1 , self.total_impulsion_angle  + 1 ) :
                numero_sequence = (impulsion + self.numero_sequence_depart - 1)  % 8 + 1
                self.mon_connecteur.envoyerImpulsion(self.SEQUENCE_PHASE[sens_rotation][numero_sequence],  numero_sequence)
                sleep( self._courbe_de_vitesse_en_seconde(impulsion ))
        if angle < 0:
            sens_rotation = 1
            for impulsion in range (1 , self.total_impulsion_angle  + 1 ) :
                numero_sequence = 9 - ((impulsion + (9 - self.numero_sequence_depart)- 1)  % 8 + 1)
                self.mon_connecteur.envoyerImpulsion(self.SEQUENCE_PHASE[sens_rotation][numero_sequence],  numero_sequence)      
                sleep( self._courbe_de_vitesse_en_seconde(impulsion ))
        if angle == 0:
            print "Angle position =  0 ; "
        self.numero_sequence_depart = numero_sequence
        sleep(0.5)
        
    def  _courbe_de_vitesse_en_seconde(self, nbre_impulsion_encours):  
        if  self.total_impulsion_angle > 2 * self.max_impulsion_vitesse_constante :            
            if nbre_impulsion_encours < self.max_impulsion_vitesse_constante:
                tour_mn_encours = (self.tour_mn_apres_reduction_maxi - self.tour_mn_apres_reduction_mini)* nbre_impulsion_encours / self.max_impulsion_vitesse_constante + self.tour_mn_apres_reduction_mini
            if (nbre_impulsion_encours >= self.max_impulsion_vitesse_constante) and (nbre_impulsion_encours <= self.total_impulsion_angle  - self.max_impulsion_vitesse_constante):
                tour_mn_encours =  self.tour_mn_apres_reduction_maxi 
            if nbre_impulsion_encours > self.total_impulsion_angle - self.max_impulsion_vitesse_constante:
                tour_mn_encours = (self.tour_mn_apres_reduction_mini - self.tour_mn_apres_reduction_maxi)*(nbre_impulsion_encours + self.max_impulsion_vitesse_constante - self.total_impulsion_angle ) /  self.max_impulsion_vitesse_constante + self.tour_mn_apres_reduction_maxi
        else:
                tour_mn_encours = self.tour_mn_apres_reduction_mini
        return 60.0 / (self.rapport_reduction_moteur * tour_mn_encours)
    
    def testLED(self, total_impulsion):
        numero_sequence = self.numero_sequence_depart_LED
        self.pause = 0.5
        if total_impulsion > 0:
            sens_rotation = 0
            for impulsion in range (1 , total_impulsion  + 1 ) :
                numero_sequence = (impulsion + self.numero_sequence_depart_LED - 1)  % 8 + 1
                self.mon_connecteur.envoyerImpulsion(self.SEQUENCE_PHASE[sens_rotation][numero_sequence],  numero_sequence)  
                print self.SEQUENCE_LED[sens_rotation][numero_sequence] + " tension " +  str(1 - numero_sequence % 2) +  " ---> CW Direction"
                sleep(self.pause)                   
        if total_impulsion < 0:
            sens_rotation = 1
            for impulsion in range (1 , abs(total_impulsion) + 1 ) :
                numero_sequence = 9 - ((impulsion + (9 - self.numero_sequence_depart_LED)- 1)  % 8 + 1)
                self.mon_connecteur.envoyerImpulsion(self.SEQUENCE_PHASE[sens_rotation][numero_sequence],  numero_sequence) 
                print self.SEQUENCE_LED[sens_rotation][numero_sequence ] +  " tension " +  str(1- numero_sequence % 2 ) + " <--- ACW Direction "
                sleep(self.pause)
        if total_impulsion == 0:
            print "total_impulsion =  0 ; "
        self.numero_sequence_depart_LED = numero_sequence
            
if __name__ == "__main__":
    pi_xel = TetePanoramique()
    carte = CarteULN2803A() 
    x1 = Connecteur(0)
    x2 = Connecteur(1)
    
    moteurB = Moteur28BYJ_48(x2, pi_xel.axe_vertical)
    moteurB.positionnerTeteAngleEnDegres(60) 
    
#    moteurB = Moteur28BYJ_48(x1, pi_xel.axe_horizontal)
#    moteurB.testLED(-11)
#    moteurB.testLED(0)
#    moteurB.testLED(13)
#    moteurB.testLED(-9)
#    
    
#    moteurB.positionnerTeteAngleEnDegres(10)
#    moteurA.positionnerTeteAngleEnDegres(-71)
#    moteurB.positionnerTeteAngleEnDegres(-58)
#    carte.arretPropre()
#    print "numero " + str(moteurVertical.numero_sequence_fin)
    
#print 'os.system("pktriggercord-cli --model=K-5 --auto_focus --file_format=DNG --exposure_mode=AV --output_file=PIxels")'
#
#print "vertical : " +  str(degrees(verticale_fov)) + " hauteur : " + str(degrees(hauteur_fov))
#print "verticale_angle_de_rotation : " + str(degrees(verticale_angle_de_rotation)) + " hauteur_angle_de_rotation : " + str(degrees(hauteur_angle_de_rotation))
#print "veticale_nombre_de_photo : " + str(veticale_nombre_de_photo) + " hauteur_nombre_de_photo : " + str(hauteur_nombre_de_photo)
